/*
	12 Player Snake Clone
					by
				Slinga

				Version 2.04

				x Dimensions - 1-38
				y Dimensions - 5-24
*/

/*
** Jo Sega Saturn Engine
** Copyright (c) 2012-2017, Johannes Fetz (johannesfetz@gmail.com)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Johannes Fetz nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL Johannes Fetz BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <jo/jo.h> // Required for basic sgl functions

struct location
{
	int x;
	int y;
	struct location* next;
};

struct food
{
	char shape[2]; // The shape of the food
	int x;
	int y;
};

struct snake
{
	int ID;
	char shape[2]; // the shape of the snake
	int dir; // current direction snake is moving in
	int active; // Is this player playing or not
	struct location* head; // points to the head of the snake
	struct location* tail; // points to the tail of the snake

	// variables for score
	int numApples;
	int numDeaths;
	int numKills;
	int score;

};

// Controller assignments
static Uint16 pad_asign[]=
{
	PER_DGT_KU,
	PER_DGT_KD,
	PER_DGT_KR,
	PER_DGT_KL,
	PER_DGT_TA,
	PER_DGT_TB,
	PER_DGT_TC,
	PER_DGT_ST,
	PER_DGT_TX,
	PER_DGT_TY,
	PER_DGT_TZ,
	PER_DGT_TR,
	PER_DGT_TL,
};

void displayText(); // Displays the heading information
void displaySMTFPresents(); // Displays Sega Saturn Multiplayer Task Force logo
void initializePlayer(struct snake* somePlayer, int controllerNum);
void drawGrid(); // Draws the playing field
void drawSnake(struct snake* somePlayer, struct snake* players); // Updates the snake, collision detection
void killSnake(struct location* snakeHead); // erases the snake, free its memory
void initializeFood(struct food* theFood, char theShape);

void drawFood(struct food* someFood, struct snake* players);
void pressStart();
Uint16 safeFood(struct food* someFood, struct snake* players); // returns a 1 if the new position of the food is "safe"
// safe means it does not overlap an exisiting snake
void clearScore(struct snake* players);
void displayScore(struct snake* players);
void clearScreen();
void redrawScreen(struct snake* players, struct food* theFood);
void titleScreen();
void calcResolution();

void jo_main(void)
{
	int i = 0;
	Uint16 data = 0;
	Sint16 slowdown = 0;
	struct snake players[12] = {0};
	struct food theFood = {0};
	char tempArray[10] = {0};

	// Initializing functions
	slInitSystem(TV_320x240, NULL, 1); // Initializes screen

	displaySSMTFPresents(); // SSMTF logo
	drawGrid(); // Draws the playing field box

	displayText();
	titleScreen();

	srand(rand()); // set the random seed
	initializeFood(&theFood, '*');
	clearScore(players);

	slowdown = 5;

	do
	{
		//calcResolution();

		// Read the 1st player controller
		data = Smpc_Peripheral[0].data;

		// Did player decrease game speed
		if((data & pad_asign[12]) == 0)
		{
			slowdown++;
			if(slowdown>20)
			{
				slowdown = 20;
			}
			sprintf(tempArray, "SD: %i  ", slowdown);
			slPrint(tempArray, slLocate(32,4));
		}

		// Did player increase game speed
		if((data & pad_asign[11]) == 0)
		{
			slowdown--;
			if(slowdown<0)
			{
				slowdown = 0;
			}

			sprintf(tempArray, "SD: %i  ", slowdown);
			slPrint(tempArray, slLocate(32,4));
		}

		// Does the user want to see the score??
		if((data & pad_asign[10]) == 0)
		{

			displayScore(players);
			pressStart();
			clearScreen();
			redrawScreen(players, &theFood);
		}


		// Does the user want to clear scores??
		if((data & pad_asign[6]) == 0)
		{

			clearScore(players);

		}

		// Check if players on tap one joined
		// Draw existing players
		for(i = 0; i < 6; i++)
		{
			data = Smpc_Peripheral[i].data;

			if((data & pad_asign[4]) == 0 && players[i].active == 0)
			{
				initializePlayer(&players[i], i);
			}

			if(players[i].active==1)
			{
				drawSnake(&players[i], players);
			}
		}

		// Check if players on tap two joined\left
		// Draw existing players
		for(i = 6; i < 12; i++)
		{
			data = Smpc_Peripheral[i+9].data;

			if((data & pad_asign[4]) == 0 && players[i].active == 0)
			{
				initializePlayer(&players[i], i+9);
			}

			if(players[i].active==1)
			{
				drawSnake(&players[i], players);
			}
		}

		drawFood(&theFood, players);

		slSynch(); // You won't see anything without this!!
		for(i = 0; i < slowdown; i++)
		{
			slSynch(); // Slow down
		}



	}while(1);


	return;
}

void initializePlayer(struct snake* somePlayer, int controllerNum)
{
	// Create player
	if(somePlayer->active!=1)
	{
		somePlayer->ID = controllerNum;

		switch (controllerNum)
		{
			case 0:
			somePlayer->shape[0] = (char)14; // block
			break;

			case 1:
			somePlayer->shape[0] = (char)35;	// pound sign
			break;

			case 2:
			somePlayer->shape[0] = (char)23; // gaurav's gamma
			break;

			case 3:
			somePlayer->shape[0] = (char)127; // checkerboard
			break;

			case 4:
			somePlayer->shape[0] = (char)92; // looks like a V
			break;

			case 5:
			somePlayer->shape[0] = (char)38; // percent sign
			break;

			case 15:
			somePlayer->shape[0] = (char)64; // copyright symbol
			break;

			case 16:
			somePlayer->shape[0] = (char)56; // number eight
			break;

			case 17:
			somePlayer->shape[0] = (char)37; // percent sign
			break;

			case 18:
			somePlayer->shape[0] = (char)48; // zero
			break;

			case 19:
			somePlayer->shape[0] = (char)81; // letter Q
			break;

			case 20:
			somePlayer->shape[0] = (char)149; // evil snake!
			break;

			default:
			somePlayer->shape[0] = 'e';
			break;

		}

		somePlayer->shape[1] = '\0';


		if(controllerNum > 5) // Tap 2
		{

			somePlayer->head = malloc(sizeof(struct location));
			somePlayer->head->next = malloc(sizeof(struct location));

			switch(somePlayer->ID)
			{
				case 16:
				somePlayer->head->x = 9;
				somePlayer->head->y = 5;
				somePlayer->dir = 1;
				break;
				case 15:
				somePlayer->head->x = 29;
				somePlayer->head->y = 24;
				somePlayer->dir = 0;
				break;

				case 18:
				somePlayer->head->x = 29;
				somePlayer->head->y = 5;
				somePlayer->dir = 1;
				break;

				case 17:
				somePlayer->head->x = 9;
				somePlayer->head->y = 24;
				somePlayer->dir = 0;
				break;

				case 20:
				somePlayer->head->x = 19;
				somePlayer->head->y = 5;
				somePlayer->dir = 1;
				break;

				case 19:
				somePlayer->head->x = 19;
				somePlayer->head->y = 24;
				somePlayer->dir = 0;
				break;
			}

			somePlayer->tail = somePlayer->head->next->next;


			// Initiate rest of snake to offscreen positions
			somePlayer->head->next->x = 50;
			somePlayer->head->next->y = 50;
			somePlayer->head->next->next=malloc(sizeof(struct location));

			somePlayer->head->next->next->x = 50;
			somePlayer->head->next->next->y = 50;
			somePlayer->head->next->next->next = NULL;

			somePlayer->tail = somePlayer->head->next->next;


			// Draw the starting position of the snake
			slPrint(somePlayer->shape, slLocate(somePlayer->head->x, somePlayer->head->y));

			somePlayer->ID = controllerNum;
			somePlayer->active = 1;


		}
		else // Tap 1
		{
			// Player is in first port

			somePlayer->head = malloc(sizeof(struct location));
			somePlayer->head->next = malloc(sizeof(struct location));



			switch(somePlayer->ID)
			{
				case 0:
				somePlayer->head->x = 1;
				somePlayer->head->y = 9;
				somePlayer->dir = 2;
				break;
				case 1:
				somePlayer->head->x = 38;
				somePlayer->head->y = 21;
				somePlayer->dir = 3;
				break;

				case 2:
				somePlayer->head->x = 1;
				somePlayer->head->y = 21;
				somePlayer->dir = 2;
				break;

				case 3:
				somePlayer->head->x = 38;
				somePlayer->head->y = 21;
				somePlayer->dir = 3;
				break;

				case 4:
				somePlayer->head->x = 1;
				somePlayer->head->y = 15;
				somePlayer->dir = 2;
				break;

				case 5:
				somePlayer->head->x = 38;
				somePlayer->head->y = 15;
				somePlayer->dir = 3;
				break;
			}

			// Initiate rest of snake to offscreen positions
			somePlayer->head->next->x = 50;
			somePlayer->head->next->y = 50;
			somePlayer->head->next->next=malloc(sizeof(struct location));

			somePlayer->head->next->next->x = 50;
			somePlayer->head->next->next->y = 50;
			somePlayer->head->next->next->next = NULL;

			somePlayer->tail = somePlayer->head->next->next;


			// Draw the starting position of the snake
			slPrint(somePlayer->shape, slLocate(somePlayer->head->x, somePlayer->head->y));

			somePlayer->ID = controllerNum;
			somePlayer->active = 1;
		}
	}
	else // destroy player
	{
		// erase old paddle position
		//slPrint(" ", slLocate(somePlayer->head->x, somePlayer->head->y));
		killSnake(somePlayer->head);
		// need to implement death function

		// You died, so increase your deaths
		// Decrease your score
		somePlayer->numDeaths++;
		somePlayer->score--;

		somePlayer->active = 0;
		somePlayer->head->x = -100;
		somePlayer->head->y = -100;
		somePlayer->dir = 0;

	}
}

void drawGrid()
{
	int j;

	char temp[10];
	char top[40]; // The top of the playing field
	char bottom[40]; // The bottom of the playing field

	char topLeftPit[2];
	char bottomRightPit[2];
	char topRightPit[2];
	char bottomLeftPit[2];
	char sidePit[2];
	char topPit[2];

	// Fill the arrays with '-'
	sprintf(temp, "%c", 21);
	for(j = 0; j<38; j++)
	{
		top[j] = temp[0];
		bottom[j] = temp[0];
	}

	// Corner pieces
	sprintf(temp, "%c", 23);
	top[0] = temp[0];

	sprintf(temp, "%c", 24);
	top[37] = temp[0];

	sprintf(temp, "%c", 25);
	bottom[0] = temp[0];

	sprintf(temp, "%c", 26);
	bottom[37] = temp[0];

	top[38] = '\0';
	bottom[38] = '\0';

	// Draw the top and bottom borders
	slPrint(top, slLocate(1, 5));
	slPrint(bottom, slLocate(1, 24));

	// Draw the sides
	sprintf(temp, "%c", 22);
	for(j = 6; j<24; j++)
	{
		slPrint(temp, slLocate(1, j));
		slPrint(temp, slLocate(38, j));

	}

	// Draw the snake pits
	sprintf(temp, "%c", 23);
	topLeftPit[0] = temp[0];
	topLeftPit[1] = '\0';

	sprintf(temp, "%c", 25);
	bottomLeftPit[0] = temp[0];
	bottomLeftPit[1] = '\0';

	sprintf(temp, "%c", 24);
	topRightPit[0] = temp[0];
	topRightPit[1] = '\0';

	sprintf(temp, "%c", 26);
	bottomRightPit[0] = temp[0];
	bottomRightPit[1] = '\0';

	sprintf(temp, "%c", 22);
	sidePit[0] = temp[0];
	sidePit[1] = '\0';

	sprintf(temp, "%c", 21);
	topPit[0] = temp[0];
	topPit[1] = '\0';

	// Left Side Pits
	slPrint(bottomRightPit, slLocate(1,8));
	slPrint(topLeftPit, slLocate(0,8));
	slPrint(sidePit, slLocate(0,9));
	slPrint(" ", slLocate(1,9));
	slPrint(bottomLeftPit, slLocate(0,10));
	slPrint(topRightPit, slLocate(1,10));

	slPrint(bottomRightPit, slLocate(1,14));
	slPrint(topLeftPit, slLocate(0,14));
	slPrint(sidePit, slLocate(0,15));
	slPrint(" ", slLocate(1,15));
	slPrint(bottomLeftPit, slLocate(0,16));
	slPrint(topRightPit, slLocate(1,16));

	slPrint(bottomRightPit, slLocate(1,20));
	slPrint(topLeftPit, slLocate(0,20));
	slPrint(sidePit, slLocate(0,21));
	slPrint(" ", slLocate(1,21));
	slPrint(bottomLeftPit, slLocate(0,22));
	slPrint(topRightPit, slLocate(1,22));

	// Right Side Pits
	slPrint(bottomLeftPit, slLocate(38,8));
	slPrint(topRightPit, slLocate(39,8));
	slPrint(sidePit, slLocate(39,9));
	slPrint(" ", slLocate(38,9));
	slPrint(bottomRightPit, slLocate(39,10));
	slPrint(topLeftPit, slLocate(38,10));

	slPrint(bottomLeftPit, slLocate(38,14));
	slPrint(topRightPit, slLocate(39,14));
	slPrint(sidePit, slLocate(39,15));
	slPrint(" ", slLocate(38,15));
	slPrint(bottomRightPit, slLocate(39,16));
	slPrint(topLeftPit, slLocate(38,16));

	slPrint(bottomLeftPit, slLocate(38,20));
	slPrint(topRightPit, slLocate(39,20));
	slPrint(sidePit, slLocate(39,21));
	slPrint(" ", slLocate(38,21));
	slPrint(bottomRightPit, slLocate(39,22));
	slPrint(topLeftPit, slLocate(38,22));

	// Top Pits
	slPrint(bottomRightPit, slLocate(8,5));
	slPrint(topLeftPit, slLocate(8,4));
	slPrint(topPit, slLocate(9,4));
	slPrint(" ", slLocate(9,5));
	slPrint(topRightPit, slLocate(10,4));
	slPrint(bottomLeftPit, slLocate(10,5));

	slPrint(bottomRightPit, slLocate(18,5));
	slPrint(topLeftPit, slLocate(18,4));
	slPrint(topPit, slLocate(19,4));
	slPrint(" ", slLocate(19,5));
	slPrint(topRightPit, slLocate(20,4));
	slPrint(bottomLeftPit, slLocate(20,5));

	slPrint(bottomRightPit, slLocate(28,5));
	slPrint(topLeftPit, slLocate(28,4));
	slPrint(topPit, slLocate(29,4));
	slPrint(" ", slLocate(29,5));
	slPrint(topRightPit, slLocate(30,4));
	slPrint(bottomLeftPit, slLocate(30,5));


	// Bottom Pits
	slPrint(topRightPit, slLocate(8,24));
	slPrint(bottomLeftPit, slLocate(8,25));
	slPrint(topPit, slLocate(9,25));
	slPrint(" ", slLocate(9,24));
	slPrint(bottomRightPit, slLocate(10,25));
	slPrint(topLeftPit, slLocate(10,24));

	slPrint(topRightPit, slLocate(18,24));
	slPrint(bottomLeftPit, slLocate(18,25));
	slPrint(topPit, slLocate(19,25));
	slPrint(" ", slLocate(19,24));
	slPrint(bottomRightPit, slLocate(20,25));
	slPrint(topLeftPit, slLocate(20,24));

	slPrint(topRightPit, slLocate(28,24));
	slPrint(bottomLeftPit, slLocate(28,25));
	slPrint(topPit, slLocate(29,25));
	slPrint(" ", slLocate(29,24));
	slPrint(bottomRightPit, slLocate(30,25));
	slPrint(topLeftPit, slLocate(30,24));


}


void drawSnake(struct snake* someSnake, struct snake* players)
{
	Uint16 data;
	Uint16 i;
	struct location* temp;
	struct location* temp2;

	// Checks the controller for input
	data = Smpc_Peripheral[someSnake->ID].data;

	// Check vertical movement
	if((data & pad_asign[1])== 0)
	{
		if(someSnake->dir != 0)
		{
			someSnake->dir = 1;
		}
	}
	else if((data & pad_asign[0])== 0)
	{
		if(someSnake->dir!= 1)
		{
			someSnake->dir = 0;
		}
	}
	// Check horizontal movement
	else if((data & pad_asign[2])== 0)
	{
		if(someSnake->dir!= 3)
		{
			someSnake->dir = 2;
		}
	}
	else if((data & pad_asign[3])== 0)
	{
		if(someSnake->dir!=2)
		{
			someSnake->dir = 3;
		}
	}

	// erase old snake position
	// need it so that it deletes only the tail
	temp = (someSnake->head);

	// Temp will be second to last node
	while(temp->next!=someSnake->tail)
	{
		temp = temp->next;

	}

	// Erase the old tail
	slPrint(" ", slLocate(someSnake->tail->x, someSnake->tail->y));
	temp->next = NULL;

	// Calc snake's new position
	if(someSnake->dir==0)
	{
		// up
		someSnake->tail->y = someSnake->head->y - 1;
		someSnake->tail->x = someSnake->head->x;
	}
	else if(someSnake->dir==1)
	{
		// down
		someSnake->tail->y = someSnake->head->y + 1;
		someSnake->tail->x = someSnake->head->x;
	}
	else if(someSnake->dir==2)
	{
		// right
		someSnake->tail->x = someSnake->head->x + 1;
		someSnake->tail->y = someSnake->head->y;
	}
	else if(someSnake->dir==3)
	{
		// left
		someSnake->tail->x = someSnake->head->x-1;
		someSnake->tail->y = someSnake->head->y;
	}


	// Check collision with ceiling\floor
	if(someSnake->tail->y <= 5 || someSnake->tail->y >= 24)
	{
		// destroy
		if(someSnake->dir==0)
		{
			// up
			someSnake->tail->y = someSnake->head->y +1;
			someSnake->tail->x = someSnake->head->x;
		}
		else if(someSnake->dir==1)
		{
			// down
			someSnake->tail->y = someSnake->head->y - 1;
			someSnake->tail->x = someSnake->head->x;
		}

		initializePlayer(someSnake, someSnake->ID);
		return;
	}

	// Check collision with walls
	if(someSnake->tail->x <= 1 || someSnake->tail->x >= 38)
	{
		// destroy
		if(someSnake->dir==2)
		{
			// right
			someSnake->tail->x =  someSnake->head->x - 1;
			someSnake->tail->y = someSnake->head->y;
		}
		else if(someSnake->dir==3)
		{
			// left
			someSnake->tail->x =  someSnake->head->x+1;
			someSnake->tail->y = someSnake->head->y;
		}

		initializePlayer(someSnake, someSnake->ID);
		return;

	}

	for(i = 0; i < 12; i++)
	{
		if(players[i].active==1)
		{

			// Don't check for collisions with your own head!!
			if(someSnake->ID!=i)
			{
				temp2 = players[i].head;
			}
			else
			{
				temp2 = players[i].head->next;

			}

			while(temp2!=NULL)
			{
				// check if there was a collision
				if(someSnake->tail->x == temp2->x && someSnake->tail->y == temp2->y)
				{




					/*  head on collision detection not working???s
					// Check if it was a head on collision
					if(temp2==players[i].head)
					{
					struct snake* temp = &players[i];

					initializePlayer(temp, temp->ID);
					//initializePlayer(someSnake, someSnake->ID);
					return;

					// You guys ran into each other, no points for anybody!!

				}
				*/

				initializePlayer(someSnake, someSnake->ID);

				if(someSnake->ID!=players[i].ID)
				{
					// Other player killed you, reward him
					players[i].numKills++;
					players[i].score++;
				}

				return;

			}

			temp2= temp2->next;

		}
	}
}


// set the new tail
someSnake->tail->next = someSnake->head;
someSnake->head = someSnake->tail;
someSnake->tail = temp;
someSnake->tail->next = NULL;
temp = NULL;
temp2 = NULL;

// draw new snake position
// draws only the new head
slPrint(someSnake->shape, slLocate(someSnake->head->x, someSnake->head->y));



return;
}

// Displays the "Sega Saturn Multiplayer Task Force" presents screen
void displaySSMTFPresents()
{
	Uint16 counter = 0;

	do{
		slPrint("The Sega Saturn", slLocate(13, 11));
		slPrint("Multiplayer Task Force", slLocate(9, 14));
		slPrint("Proudly Presents", slLocate(12, 17));

		slSynch();

		counter++;

	}while(counter < 200);

	clearScreen();
}


void displayText()
{
	slPrint("Twelve Snakes Version 2.04 by Slinga", slLocate(1,1));
	slPrint("Press A to join", slLocate(1,2));
}

void calcResolution()
{
	char temp[100];


	for(int i = 0; i < 54; i++)
	{
		sprintf(temp, "H_%d", i);
		slPrint(temp, slLocate(5, i));
	}

	for(int i = 0; i < 60; i++)
	{
		sprintf(temp, "V");
		slPrint(temp, slLocate(i, 20));


		sprintf(temp, "_");
		slPrint(temp, slLocate(i, 20+1));

		sprintf(temp, "%d", i/10);
		slPrint(temp, slLocate(i, 20+2));

		sprintf(temp, "%d", i%10);
		slPrint(temp, slLocate(i, 20+3));
	}
}


// Displays the text "Press Start" and waits for the user to hit start
void pressStart()
{
	Uint16 start;
	do{
		start = Smpc_Peripheral[0].data; // Checks if start button has been pressed
		slPrint("Press Start", slLocate(15, 22));
		slSynch();
		slSynch();
		slSynch();

	}while((start & pad_asign[7])!= 0);

}

void killSnake(struct location* snakeHead)
{
	if(snakeHead->next!=NULL)
	{
		killSnake(snakeHead->next);
	}

	slPrint(" ", slLocate(snakeHead->x, snakeHead->y));
	//free(snakeHead); // gives me weird errors
}

void initializeFood(struct food* theFood, char theShape)
{
	// Shape, and initial position of food
	theFood->shape[0] = theShape;
	theFood->shape[1] = '\0';
	theFood->x = 19;
	theFood->y = 15;

	// Print the food
	slPrint(theFood->shape, slLocate(theFood->x, theFood->y));
}

void drawFood(struct food* theFood, struct snake* players)
{
	struct location* temp;
	Uint16 i;

	for(i = 0; i < 12; i++)
	{
		if(players[i].active==1)
		{
			temp = players[i].head;

			// check if there was a collision
			if(theFood->x == temp->x && theFood->y == temp->y)
			{
				srand(players[i].tail->x + players[i].tail->y);

				do{

					theFood->x = rand()%36 + 2;
					theFood->y = rand()%18 + 6;
				}while(safeFood(theFood, players)!=1);

				slPrint(theFood->shape, slLocate(theFood->x, theFood->y));

				// Add a new segment, make that segment the tail
				players[i].tail->next = malloc(sizeof(struct location));
				players[i].tail->next->x = players->tail->x;
				players[i].tail->next->y = players->tail->y;
				players[i].tail->next->next = NULL;
				players[i].tail = players[i].tail->next;

				// You ate the apple, increase your score
				players[i].numApples++;
				players[i].score++;


				return;
			}
		}
	}
}

Uint16 safeFood(struct food* someFood, struct snake* players)
{
	Uint16 i;
	struct location* temp;

	for(i = 0; i<12; i++)
	{
		if(players[i].active==1)
		{

			temp = players[i].head;

			while(temp!=NULL)
			{
				// check if there was a collision
				if(someFood->x == temp->x && someFood->y == temp->y)
				{
					return 0;
				}
				temp = temp->next;
			}

		}

	}

	return 1;

}

void clearScore(struct snake* players)
{
	Uint16 i;

	for(i = 0; i < 12; i++)
	{
		players[i].numApples = 0;
		players[i].numDeaths = 0;
		players[i].numKills = 0;
		players[i].score = 0;
	}


}

void displayScore(struct snake* players)
{
	Uint16 i;
	char temp[50];
	Uint16 counter = 7;

	temp[0] = '\0';

	slPrint("P#    A#    K#    D#    Score", slLocate(5,counter++));
	slPrint("-----------------------------", slLocate(5,counter++));
	counter++;

	for(i = 0; i < 12; i++)
	{
		if(players[i].active == 1)
		{
			sprintf(temp, "%i %s%s%s     %i     %i     %i      %i", i+1, players[i].shape, players[i].shape, players[i].shape, players[i].numApples, players[i].numKills, players[i].numDeaths, players[i].score);
			slPrint(temp, slLocate(5,counter));
			counter++;

		}
	}



}


void clearScreen()
{

	Uint16 i;

	for(i=6; i < 24; i++)
	{

		slPrint("                                   ", slLocate(3,i));
	}



}

void redrawScreen(struct snake* players, struct food* theFood)
{

	Uint16 i = 0;
	struct location* temp = NULL;

	for(i =0; i <12; i++)
	{
		// redraw only the active players
		if(players[i].active==1)
		{
			temp = players[i].head;

			while(temp!=NULL)
			{
				slPrint(players[i].shape, slLocate(temp->x, temp->y));
				temp = temp->next;

			}

		}

	}

	slPrint(theFood->shape, slLocate(theFood->x, theFood->y));

}


void titleScreen()
{
	slPrint("TTTTTT", slLocate(3,7));
	slPrint("  TT  ", slLocate(3,8));
	slPrint("  TT  ", slLocate(3,9));
	slPrint("  TT  ", slLocate(3,10));
	slPrint("  TT  ", slLocate(3,11));
	slPrint("  TT  ", slLocate(3,12));

	slPrint("W     W", slLocate(10,7));
	slPrint("W     W", slLocate(10,8));
	slPrint("W  W  W", slLocate(10,9));
	slPrint("WW W WW", slLocate(10,10));
	slPrint(" WWWWW ", slLocate(10,11));
	slPrint(" WW WW ", slLocate(10,12));

	slPrint("EEEE", slLocate(18,7));
	slPrint("E   ", slLocate(18,8));
	slPrint("EEEE", slLocate(18,9));
	slPrint("EEEE", slLocate(18,10));
	slPrint("E   ", slLocate(18,11));
	slPrint("EEEE", slLocate(18,12));

	slPrint("L  ", slLocate(23,7));
	slPrint("L  ", slLocate(23,8));
	slPrint("L  ", slLocate(23,9));
	slPrint("L  ", slLocate(23,10));
	slPrint("L  ", slLocate(23,11));
	slPrint("LLL", slLocate(23,12));

	slPrint("V   V", slLocate(26,7));
	slPrint("V   V", slLocate(26,8));
	slPrint("V   V", slLocate(26,9));
	slPrint("VV VV", slLocate(26,10));
	slPrint(" V V ", slLocate(26,11));
	slPrint(" VVV ", slLocate(26,12));

	slPrint("EEEE", slLocate(32,7));
	slPrint("E   ", slLocate(32,8));
	slPrint("EEEE", slLocate(32,9));
	slPrint("EEEE", slLocate(32,10));
	slPrint("E   ", slLocate(32,11));
	slPrint("EEEE", slLocate(32,12));

	slPrint("SSSS", slLocate(4,14));
	slPrint("S   ", slLocate(4,15));
	slPrint("S   ", slLocate(4,16));
	slPrint("SSSS", slLocate(4,17));
	slPrint("   S", slLocate(4,18));
	slPrint("   S", slLocate(4,19));
	slPrint("SSSS", slLocate(4,20));

	slPrint("N   N", slLocate(9,14));
	slPrint("NN  N", slLocate(9,15));
	slPrint("N N N", slLocate(9,16));
	slPrint("N NNN", slLocate(9,17));
	slPrint("N  NN", slLocate(9,18));
	slPrint("N   N", slLocate(9,19));
	slPrint("N   N", slLocate(9,20));

	slPrint("  A  ", slLocate(15,14));
	slPrint(" AAA ", slLocate(15,15));
	slPrint("AA AA", slLocate(15,16));
	slPrint("A   A", slLocate(15,17));
	slPrint("AAAAA", slLocate(15,18));
	slPrint("A   A", slLocate(15,19));
	slPrint("A   A", slLocate(15,20));

	slPrint("K  K", slLocate(21,14));
	slPrint("K  K", slLocate(21,15));
	slPrint("K K ", slLocate(21,16));
	slPrint("KKK ", slLocate(21,17));
	slPrint("K K ", slLocate(21,18));
	slPrint("K  K", slLocate(21,19));
	slPrint("K  K", slLocate(21,20));

	slPrint("EEEE", slLocate(26,14));
	slPrint("E   ", slLocate(26,15));
	slPrint("E   ", slLocate(26,16));
	slPrint("EEEE", slLocate(26,17));
	slPrint("E   ", slLocate(26,18));
	slPrint("E   ", slLocate(26,19));
	slPrint("EEEE", slLocate(26,20));

	slPrint("SSSS", slLocate(31,14));
	slPrint("S   ", slLocate(31,15));
	slPrint("S   ", slLocate(31,16));
	slPrint("SSSS", slLocate(31,17));
	slPrint("   S", slLocate(31,18));
	slPrint("   S", slLocate(31,19));
	slPrint("SSSS", slLocate(31,20));

	slPrint("Dedicated to the man with one knee", slLocate(3,23));



	pressStart();
	clearScreen();


}
